import { EncryptionFunction, DecryptionFunction } from '../native/JSBridge.js';
import axios from "axios";
import Qs from "qs";
import Toast from '../../toast';
import { stroageData, getDataFromStroageData } from './demoStroage';
axios.defaults.headers.post['Content-Type'] = 'application/x-www-form-urlencoded';
axios.defaults.timeout = 90000; //请求超时时间
axios.defaults.transformRequest = [function (data) {
    // 做任何你想要的数据转换
    return Qs.stringify(data);
}];

//添加响应拦截器
axios.interceptors.response.use(function (response) {
    //对响应数据做些事
    return response.data;
}, function (error) {
    //请求错误时做些事
    return Promise.reject(error);
});

/**
 * post请求(data加密)
 * @param url
 * @param data
 */
export function doPostRequest(url) {
    var data = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    var p = new Promise(function (res, rej) {
        var cryptKey = "";
        var zip = 1;
        var keyType = 24;

        // start mj 2019-09-23 yqtdemo 缓存请求数据，具体哪些缓存，接口内部处理
        // 如果是demo环境，直接从缓存取数据返回
        if (data.isDemo === 1) {
            var demoResData = getDataFromStroageData(url, data);
            // 如果缓存有数据而且是需要缓存的数据,直接往上抛，否则还是走网络请求
            if (demoResData && demoResData.cacheKey !== "noCache") {
                var retData = demoResData.resData;
                // 重要:
                // 如果连code都没有，说明是缓存中没有数据，需要toast提示，方便调试检查没有缓存到该缓存的数据
                // 上线时建议干掉这句showToast
                if (!Object.prototype.hasOwnProperty.call(retData, 'code')) {
                    // showToast("该缓存的数据没缓存: " + url);
                }
                console.log("Net res ," + url + ",", Number(new Date()));
                // res(retData);
                setTimeout(function () {
                    console.log("Net res timeout," + url + ",", Number(new Date()));
                    res(retData);
                }, 350);
                return;
            }
        }
        // end mj 2019-09-23 yqtdemo 缓存请求数据，具体哪些缓存，接口内部处理

        encryptData(data).then(function (encryptData) {
            // 加密request数据
            cryptKey = encryptData.key;
            zip = encryptData.zip;
            keyType = encryptData.keyType;
            // 兼容mPaaS网关请求处理， 如果是mPaaS浏览器，直接走mPaaS网关请求
            //  if (window.AlipayJSBridge) {
            //     console.log("doPostRequest -- window.AlipayJSBridge...");
            //     return aliCallRequest(url, encryptData);
            // }
            console.log("doPostRequest -- axios request...");
            return sendPost('POST', url, encryptData, 'json'); // 发送网络请求
        }).then(function (response) {
            // 服务器response处理
            return decryptData(response, cryptKey, zip, keyType); // 去解密
        }).then(function (decryptData) {
            // 解密成功(包含code==0和code！=0，由业务自己处理code)
            // start mj 2019-09-23 yqtdemo 缓存请求数据，具体哪些缓存，接口内部处理
            stroageData(url, data, decryptData);
            // end mj 2019-09-23 yqtdemo 缓存请求数据，具体哪些缓存，接口内部处理
            res(decryptData);
        })["catch"](function (e) {
            // 统一处理异常情况, 由调用层自己处理异常情况
            console.log("formRequest e = " + JSON.stringify(e));
            rej(e);
        });
    });
    return p;
}

/**
 * 加密数据
 */
function encryptData() {
    var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    var p = new Promise(function (res, rej) {
        EncryptionFunction(data).then(function (eData) {
            var d = {
                bdata: JSON.stringify({
                    data: eData.encryptData,
                    zip: eData.zip,
                    key: eData.key,
                    keyType: eData.keyType
                }),
                key: eData.key,
                zip: eData.zip,
                keyType: eData.keyType
            };
            //          DecryptionFunction(JSON.parse(d.bdata)).then(function (result) {
            //              console.log("加密后："+JSON.stringify(result));
            //          });
            res(d);
        });
    });
    return p;
}

/**
 * 解密数据
 */
function decryptData(result, key, zip, keyType) {
    var p = new Promise(function (res, rej) {
        var response = result;
        if ('string' === typeof result) {
            response = JSON.parse(result);
        }
        // ssp服务的response最外层协议结构
        // mj 2019-04-17 协议新增ecode，emsg; 用于拼接在desc后面展示给用户
        var ret = {
            code: response.code,
            ecode: response.ecode,
            emsg: response.emsg,
            desc: response.rdesc,
            data: response.data
        };
        if (isSuccess(response) && !!response.data) {
            // code == 0并且data不能为空
            var d = {
                zip: zip,
                key: key,
                keyType: keyType,
                data: response.data
            };
            DecryptionFunction(d).then(function (result) {
                ret.data = result;
                res(ret);
            });
        } else {
            // 服务器返回异常code
            res(ret);
        }
    });
    return p;
}

/**
 * result 是否成功 code == 0
 * @param result
 * @returns {boolean}
 */
export function isSuccess(result) {
    if (result && result.code === 0) {
        return true;
    } else {
        return false;
    }
}

/**
 * 网络请求
 * @param method
 * @param url
 * @param data
 * @param responseType
 * @returns {Promise<any>}
 */
function sendPost(method, url, data, responseType) {

    function getClass(object) {
        return Object.prototype.toString.call(object).match(/^\[object\s(.*)\]$/)[1];
    }
    return new Promise(function (res, rej) {
        var tempData = '',
            sendData = null;
        for (var o in data) {
            if (data.hasOwnProperty(o)) {
                tempData += "&" + o + "=" + data[o];
            }
        }
        tempData = tempData.slice(1);
        var xhr = new window.XMLHttpRequest();
        if (method === 'GET') {
            //如果是GET请求需要把data拼接到url中
            var and = url.indexOf('?') === -1 ? '?' : '&';
            url += and + tempData;
        } else {
            //否则直接发送
            sendData = tempData;
        }
        xhr.open(method, url);
        xhr.timeout = 30000;
        xhr.ontimeout = function (event) {
            console.log("sendPost - ontimeout..." + event);
            var errInfo = {
                errorCode: "100001",
                errorDesc: "请求超时"
            };
            rej(errInfo);
        };
        xhr.setRequestHeader('content-type', 'application/x-www-form-urlencoded;');
        if ('json' !== responseType && 'JSON' !== responseType) {
            xhr.responseType = responseType || 'text';
        }
        xhr.onreadystatechange = function (e) {
            if (e.currentTarget.readyState === 4) {
                if (e.currentTarget.status === 200) {
                    if ('json' === responseType || 'JSON' === responseType) {
                        var returnRes = e.currentTarget.response;
                        try {
                            if (getClass(returnRes) === 'Object') {
                                res(returnRes);
                            } else if (getClass(returnRes) === 'String') {
                                res(JSON.parse(returnRes));
                            }
                        } catch (e) {
                            res(returnRes);
                        }
                    } else if ('arraybuffer' === responseType) {
                        //将blob数据类型改为arraybuffer兼容Android4.3
                        res(xhr.response);
                    }
                } else {
                    console.log("formRequest - xhr.status is not 200");
                    var errInfo = {
                        errorCode: "100002",
                        errorDesc: "网络异常" + e.currentTarget.status
                    };
                    rej(errInfo);
                }
            }
        };
        xhr.send(sendData);
    });
}

//AliCall 请求统一处理 20190307 zw
export function aliCallRequest(url, data) {
    return new Promise(function (res, rej) {
        // 通过url解析mPaaS网关api名称、path参数
        var mgwInfo = parseMGRParam(url, "attendance");

        // 补充mPaaS配置参数
        data["apiName"] = mgwInfo.apiName; // 接口名称 xxx.do,mPaaS网关api上的path参数
        data["cryptPostFlag"] = true; // 数据加密

        var params = [];
        params.push(data);
        console.log('003.params:', params);
        AlipayJSBridge.call('rpc', {
            operationType: mgwInfo.apiOptType, // mPaaS网关api名称
            requestData: params, // 业务参数
            headers: {
                'content-type': 'application/x-www-form-urlencoded;'
            },
            timeout: 90
        }, function (result) {
            console.log('002 mgwInfo: ' + JSON.stringify(mgwInfo) + ', result- :' + JSON.stringify(result));
            // rpc 本身异常
            if (result.errorMessage) {
                Toast(result.errorMessage || '请求失败 ali rpc error');
                var errInfo = {
                    errorCode: result.error,
                    errorDesc: result.errorMessage
                };
                rej(errInfo);
            } else {
                // 不是rpc异常，就处理业务返回码
                res(result);
            }
        });
    })["catch"](function (error) {
        console.log("formRequest Error = " + JSON.stringify(error));
    });
}
/**
 * @description: 从URL解析网关请求的参数
 * @param url：请求地址; appServer 应用服务名称（如 approval）
 * @return:
 * @Author: miaoju
 * @Date: 2019-06-26 00:19:25
 */
function parseMGRParam(url, appServer) {
    var ret = {
        apiName: "", // 接口名称 ： home/bankoutlets/downloadBankList.do
        apiOptType: "" // 网关配置的接口 attendanceAction
    };
    if (url) {
        // 1、解析mPaaS网关api的operationType
        var appName = parseMGWOptType(url, appServer);
        ret.apiOptType = appName + "Action";
        //2、解析api接口参数 apiName
        ret.apiName = parseApiName(url, appName);
    }
    return ret;
}

/**
 * 解析swp的请求url，路径里面都是带home的，取url中home/后面的一段地址作为 apiName
 * url例子： https://yqtdev.sinosun.com:18094/yqt/bankOutlets/home/bankoutlets/downloadBankList.do
 * 返回：home/bankoutlets/downloadBankList.do
 */
function parseApiName(url, appName) {
    var apiName = url;
    var micAppName = appName + "/";
    if (url) {
        var pos = url.indexOf(micAppName);
        if (pos > -1) {
            // 取url中home/后面的一段地址作为 apiName
            apiName = url.substr(pos + micAppName.length);
        }
    }
    return apiName;
}

/**
 * 解析url,获取mPaaS网关的api的operationType, 服务名+Action
 * url例子： https://yqtdev.sinosun.com:18094/yqt/bankOutlets/home/bankoutlets/downloadBankList.do
 * 返回：bankOutletsAction
 */
function parseMGWOptType(url, appServer) {
    var apiOptType = appServer;
    // 如果url包含了/yqt/，后面就是服务名称
    if (url && url.indexOf("/yqt/") > -1) {
        var subUrl = url.substr(url.indexOf("/yqt/") + "/yqt/".length);
        apiOptType = subUrl.substr(0, subUrl.indexOf("/"));
    }
    return apiOptType;
}